############################################################################
#                                                                          #
# Copyright (c)2009, 2010, Digi International (Digi). All Rights Reserved. #
#                                                                          #
# Permission to use, copy, modify, and distribute this software and its    #
# documentation, without fee and without a signed licensing agreement, is  #
# hereby granted, provided that the software is used on Digi products only #
# and that the software contain this copyright notice,  and the following  #
# two paragraphs appear in all copies, modifications, and distributions as #
# well. Contact Product Management, Digi International, Inc., 11001 Bren   #
# Road East, Minnetonka, MN, +1 952-912-3444, for commercial licensing     #
# opportunities for non-Digi products.                                     #
#                                                                          #
# DIGI SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED   #
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          #
# PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, #
# PROVIDED HEREUNDER IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. #
# DIGI HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,         #
# ENHANCEMENTS, OR MODIFICATIONS.                                          #
#                                                                          #
# IN NO EVENT SHALL DIGI BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,      #
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,   #
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF   #
# DIGI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                #
#                                                                          #
############################################################################

import logging
import time
from datetime import datetime, timedelta

from google.appengine.ext import webapp
from google.appengine.ext.webapp import util
from google.appengine.ext import db

from models import User
from idigi import Sample, iDigiClient

class CollectHandler(webapp.RequestHandler):
    # Periodically gathers samples from enabled users' gateways and
    # stores them in the datastore.
    def get(self):
        # Find users with active data-logging
        users = User.gql('WHERE logging = :1', True)

        for user in users:
            try: 
                # Get samples from a channel dump
                idigi_client = iDigiClient(username=user.username,
                                           password=user.password)

                samples = idigi_client.channel_dump(device_id=user.gateway)

                # Add user key to Samples and save in Datastore
                for sample in samples:
                    if sample.channel_name.endswith('light') \
                       or sample.channel_name.endswith('temperature') \
                       or sample.channel_name.endswith('current'):
                        # Save samples we care about
                        sample.user = user
                        sample.put()

            except Exception, e:
                logging.error(
                    "Exception while processing data collection for %s" % 
                    user.username)
                logging.error(str(e))

            if user.logging_deadline and user.logging_deadline < datetime.now():
                logging.debug('Turning logging off for user %s' %
                              user.username)
                user.logging = False
                user.put()


class CleanupHandler(webapp.RequestHandler):
    def get(self):
        # Cleanup samples more than a day old
        query = Sample.gql('WHERE internal_ts < :1',
                           datetime.now() - timedelta(1))

        for item in query:
            item.delete()


def main():
    # use the controllers, plus hardcoded "/" to the login handler
    application = webapp.WSGIApplication([
        ('/tasks/collect', CollectHandler),
        ('/tasks/cleanup', CleanupHandler),
        ],
        debug=True)
    
    util.run_wsgi_app(application)

if __name__ == '__main__':
    main()
