############################################################################
#                                                                          #
# Copyright (c)2009, 2010, Digi International (Digi). All Rights Reserved. #
#                                                                          #
# Permission to use, copy, modify, and distribute this software and its    #
# documentation, without fee and without a signed licensing agreement, is  #
# hereby granted, provided that the software is used on Digi products only #
# and that the software contain this copyright notice,  and the following  #
# two paragraphs appear in all copies, modifications, and distributions as #
# well. Contact Product Management, Digi International, Inc., 11001 Bren   #
# Road East, Minnetonka, MN, +1 952-912-3444, for commercial licensing     #
# opportunities for non-Digi products.                                     #
#                                                                          #
# DIGI SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED   #
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          #
# PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, #
# PROVIDED HEREUNDER IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. #
# DIGI HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,         #
# ENHANCEMENTS, OR MODIFICATIONS.                                          #
#                                                                          #
# IN NO EVENT SHALL DIGI BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,      #
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,   #
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF   #
# DIGI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                #
#                                                                          #
############################################################################

"""
Handles authenticating against the iDigi server and storing the results
in a session variable
"""
import hashlib
import base64
import random
import logging

from google.appengine.api import memcache

from datetime import datetime
from base_handler import BaseHandler
from models import User
from idigi import iDigiRequestError

class LogoffHandler(BaseHandler):
    """Handler to log off a user and redirect to home"""
    def get(self):
        self.response.headers.add_header('Set-Cookie', 'session=;')
        self.redirect('/')
    
    def post(self):
        self.response.headers.add_header('Set-Cookie', 'session=;')
        self.redirect('/')
        
class LoginHandler(BaseHandler):
    """
    This handler will display the home view. Authentication data is
    provided via an HTTP post to this handler. If the username/password can
    successfully authenticate to the iDigi sandbox, it will login successfully.
    This handler also creates a session_id in the cookie to keep track of the
    logged in user, also setting the csrf_id to be used in forms as a
    precaution against cross site request forgeries.
    """
    def get(self):
        self.doRender('login.html')
        
    def post(self):
        username = self.request.get('username')
        password = self.request.get('password')
        gateway = self.request.get('gateway')

        try:
            user = User.get_or_create(username, password, gateway)
        except iDigiRequestError, e:
            context = {'error': str(e)}
            logging.debug("Error: %s" % str(e))
            self.doRender('login.html', context)
            return


        # if we have user here and no exception thrown from get_or_create
        # we will create a session id and register a new crsf_id
        m = hashlib.sha512()
        m.update("salt[%s]%s:%s" % (datetime.now(), username, password))
        sess = base64.b64encode(m.digest())
        m.update("%s" % random.random()) # make different from session_id
        user.csrf_id = base64.b64encode(m.digest())
        user.save()
        memcache.add(sess, user.key(), 86400)

        # set the session id, the user.csrf_id is to used within HTML forms
        # as a hidden input field to help prevent cross site request
        # forgeries
        self.response.headers.add_header('Set-Cookie','session=%s;' % sess)

        self.redirect('/logging?state=on')
