############################################################################
#                                                                          #
# Copyright (c)2008, 2009, Digi International (Digi). All Rights Reserved. #
#                                                                          #
# Permission to use, copy, modify, and distribute this software and its    #
# documentation, without fee and without a signed licensing agreement, is  #
# hereby granted, provided that the software is used on Digi products only #
# and that the software contain this copyright notice,  and the following  #
# two paragraphs appear in all copies, modifications, and distributions as #
# well. Contact Product Management, Digi International, Inc., 11001 Bren   #
# Road East, Minnetonka, MN, +1 952-912-3444, for commercial licensing     #
# opportunities for non-Digi products.                                     #
#                                                                          #
# DIGI SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED   #
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          #
# PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, #
# PROVIDED HEREUNDER IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. #
# DIGI HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,         #
# ENHANCEMENTS, OR MODIFICATIONS.                                          #
#                                                                          #
# IN NO EVENT SHALL DIGI BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,      #
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,   #
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF   #
# DIGI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                #
#                                                                          #
############################################################################

import os
import logging

from models import User
from google.appengine.ext import webapp
from google.appengine.ext.webapp import template
from google.appengine.api import memcache


class BaseHandler(webapp.RequestHandler):
    def get_user(self, verify=True):
        """
        Returns the user for a given http request. If the user is None, it will
        redirect to the login page. If verify is True, it will check the request
        parameters for a csrf_id field and check it against the user.
        """
        cache_key = self.request.cookies.get('session')
        if cache_key is not None:
            user_key = memcache.get(cache_key)
            if user_key is not None:
                user = User.get(user_key)
                if user is not None:
                    if verify and self.request.get('csrf_id') != user.csrf_id:
                        raise Exception("Authentication failure (csrf_id)")
                    else:
                        return user
            return None
        
    
    def doRender(self, tname='index.html', values={}):
        """
        This is a utility method that will find an django file in the templates
        folder, take a dictionary of value objects and write out an html file. If no
        file name is supplied, it will render the default index.html file.
        """
        temp = os.path.join(os.path.dirname(__file__),'../templates/' + tname)
        if not os.path.isfile(temp):
            return False
        
        # Make a copy of the dictionary and add the path
        newval = dict(values)
        newval['path'] = self.request.path
        
        outstr = template.render(temp, newval)
        self.response.out.write(outstr)
        return True

    def doRenderXml(self, tname='index.xml', values={}):
        """
        This is a utility method that will find a template file in the templates
        folder, take a dictionary of value objects and write out an xml file. If no
        file name is supplied, it will render the default index.xml file.
        """
        temp = os.path.join(os.path.dirname(__file__), '../templates/' + tname)
        if not os.path.isfile(temp):
            return False
        
        # Make a copy of the dictionary and add the path
        newval = dict(values)
        newval['path'] = self.request.path
        
        outstr = template.render(temp, newval)
        self.response.headers['Content-Type'] = 'text/xml'
        self.response.out.write(outstr)
        return True
