############################################################################
#                                                                          #
# Copyright (c)2009, 2010, Digi International (Digi). All Rights Reserved. #
#                                                                          #
# Permission to use, copy, modify, and distribute this software and its    #
# documentation, without fee and without a signed licensing agreement, is  #
# hereby granted, provided that the software is used on Digi products only #
# and that the software contain this copyright notice,  and the following  #
# two paragraphs appear in all copies, modifications, and distributions as #
# well. Contact Product Management, Digi International, Inc., 11001 Bren   #
# Road East, Minnetonka, MN, +1 952-912-3444, for commercial licensing     #
# opportunities for non-Digi products.                                     #
#                                                                          #
# DIGI SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED   #
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          #
# PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, #
# PROVIDED HEREUNDER IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. #
# DIGI HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,         #
# ENHANCEMENTS, OR MODIFICATIONS.                                          #
#                                                                          #
# IN NO EVENT SHALL DIGI BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,      #
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,   #
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF   #
# DIGI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                #
#                                                                          #
############################################################################

import os
import urllib
import logging

from google.appengine.ext import webapp
from google.appengine.ext.webapp import util, template
from google.appengine.ext.db import NeedIndexError
from google.appengine.api.urlfetch import DownloadError

from handlers.auth import LoginHandler, LogoffHandler
from handlers.base_handler import BaseHandler

from idigi import Sample, iDigiClient, iDigiRequestError, iDigiError
from models import User
from datetime import datetime, timedelta


class SmartplugHandler(BaseHandler):
    def get(self):
        user = self.get_user(False)
        
        onoff = self.request.get('onoff').lower().capitalize()
        if onoff not in ('On', 'Off'):
            self.error(403)
        sample = Sample(channel_name = "rpm0.power_on", value = onoff)
        idigi_client = iDigiClient(username=user.username,
                                   password=user.password)

        print "SPH.get gw: ", user.gateway
        idigi_client.channel_set(device_id=user.gateway, sample=sample)
        self.redirect('/')


B64table = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
            'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
            'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
            'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
            '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '.']

def enc1_extended(val, minimum, maximum):
    # Scale to fit
    val = val - minimum
    maximum = maximum - minimum
    
    scaled = int(4096 * val / maximum)

    if scaled >= 4096:
        return '..'
    elif scaled >= 0:
        return B64table[scaled / 64] + B64table[scaled % 64]
    else:
        return '__'

def encode_extended(vals, minimum=0, maximum=100):
    vals = map(lambda x: enc1_extended(x, minimum, maximum), vals)
    return ''.join(vals)

MAX_MAP = {
    'light': 1200.0,
    'temperature': 40.0,
    'current': 15.0,
}
STEP_MAP = {
    'light':400,
    'temperature':10,
    'current': 5,
}
PCT_XFORM_MAP = {
    'light': lambda x: x/1200.0*100,
    'temperature': lambda x: x/40.0*100,
    'current': lambda x: x/15.0*100,
}
UNIT_MAP = {
    'light': 'Brightness',
    'temperature': 'Celsius',
    'current': 'Amps'
}
COLOR_MAP = {
    'light': ('444444','DDDD33'),
    'temperature': ('0000FF','FF0000'),
    'humidity': ('FFFFFF','3333FF'),
    'current': ('444444','FF3333'),
}
LABEL_MAP = {
    'light': "Light",
    'temperature': "Temperature",
    'humidity': "Humidity",
    'current': "Current",
}
LEGEND_MAP = {
    'sensor0': 'XBee Sensor',
    'rpm0': 'XBee Smart Plug',
    'xbr0': 'XBee Wall Router'
}
COLOR_LIST = ['00688b','ff0000','00ff00']

chart_server = 1

class AboutHandler(webapp.RequestHandler):
    def get(self):
        self.response.out.write(template.render('templates/about.html', {}))

class GaugeHandler(webapp.RequestHandler):
    @staticmethod
    def render_gauge(sample):
        global chart_server
        
        value = 0.0
        unit = "undef"
        percent = 0.0

        if not sample:
            return ''

        item = sample.channel_name.split('.')[1]

        value = float(sample.value)
        if value < 0.2:
            value = 0.0
        unit = sample.unit
        if not unit:
            unit = UNIT_MAP[item]
            
        percent = value
        if item in PCT_XFORM_MAP:
            percent = PCT_XFORM_MAP[item](value)

        rng = "%0.02f,%0.02f,%f" % (0, MAX_MAP[item],STEP_MAP[item])

        template_values = {
            'name': sample.channel_name,
            'percent': urllib.quote_plus("%.1f" % percent),
            'data_label': ("%.1f %s" % (value, unit)),
            'color_start': COLOR_MAP[item][0],
            'color_end': COLOR_MAP[item][1],
            'label': LABEL_MAP[item], 
            'chart_server': chart_server,
            'range': rng
        }
        chart_server = (chart_server + 1) % 10

        path = os.path.join(os.path.dirname(__file__), 'templates/gauge.html')
        return template.render(path, template_values)
                     
    def get(self):
        gauge = self.request.get('name')

        channel_name = gauge
        q = Sample.gql("WHERE channel_name = :1 "
                        "ORDER BY timestamp DESC "
                        "LIMIT 1", channel_name)
        row = q.get()

        self.response.out.write(self.render_gauge(row))


class HistoryHandler(BaseHandler):
    @staticmethod
    def render_history(item, user, samples = 60):
        global chart_server

        data = []
        legends = []

        minimum = 0
        maximum = 1

        for dev in ['rpm0', 'sensor0', 'xbr0']:

            query = Sample.gql("WHERE channel_name = :1 "
                               "AND user = :2 "
                               "ORDER BY timestamp DESC ",
                               dev + '.' + item, user)

            try:
                vals = query.fetch(samples)
                vals.reverse()
                vals = [float(s.value) for s in vals]
            except NeedIndexError:
                return 'Data unavailable (waiting for index)'

            if len(vals) == 0:
                continue

            local_min = min(vals)
            local_max = max(vals)
            minimum = min(local_min, minimum)
            maximum = max(local_max, maximum)

            data.append(vals)
            legends.append(LEGEND_MAP[dev])

        if len(data) == 0:
            encdata = 'e:__'
            legends = 'No data'
            rng = "%0.02f,%0.02f,%f" % (0, MAX_MAP[item],STEP_MAP[item])
        else:
            encdata = []
            logging.debug("MINIMUM: %0.02f" % minimum)
            logging.debug("MAXIMUM: %0.02f" % maximum)
            for vals in data:
                encdata.append(encode_extended(vals, minimum, maximum))

            encdata = 'e:' + ','.join(encdata)
            legends = '|'.join(legends)
            rng = "%0.02f,%0.02f" % (minimum, maximum)

        colors = ','.join(COLOR_LIST[:len(encdata)])
        unit = UNIT_MAP[item]

        template_values = {
            'data': encdata,
            'label': LABEL_MAP[item],
            'legend': legends,
            'chart_server': chart_server,
            'colors': colors,
            'range': rng,
            'unit': unit
        }
        chart_server = (chart_server + 1) % 10

        path = os.path.join(os.path.dirname(__file__), 'templates/history.html')
        return template.render(path, template_values)

            
    def get(self):
        user = self.get_user(False)
        if not user:
            #self.redirect('/login')
            self.error(500)
            return

        channel_name = self.request.get('name')
        samples = self.request.get('samples')
        if not samples:
            samples = 60
        else:
            samples = int(samples)

        self.response.out.write(self.render_history(
            channel_name, user, samples))

class LoggingHandler(BaseHandler):
    def get(self):
        user = self.get_user(False)
        if not user:
            self.redirect('/login')
            return

        onoff = self.request.get('state').lower().capitalize()
        if onoff not in ('On', 'Off'):
            self.error(403)

        if onoff == 'On':
            logging.debug("Enabling logging for " + user.username)
            user.logging = True
            # Log the users data for one hour?
            user.logging_deadline = datetime.now() + timedelta(0, 3600)
        else:
            logging.debug("Disabling logging for " + user.username)
            user.logging = False
            user.logging_deadline = None

        user.put()
        self.redirect('/')

class ClearHandler(BaseHandler):
    def get(self):
        user = self.get_user(False)
        if not user:
            self.redirect('/login')
            return

        # Cleanup samples more than a day old
        query = Sample.gql('WHERE user = :1',
                           user)

        for item in query:
            item.delete()

        self.redirect('/')

class MainHandler(BaseHandler):
    def get(self):
        user = self.get_user(False)
        if not user:
            self.redirect('/login')
            return

        # Update gauges
        idigi_client = iDigiClient(username=user.username,
                                   password=user.password)

        try:
            samples = idigi_client.channel_dump(device_id=user.gateway)
        except iDigiRequestError, e:
            import traceback
            traceback.print_exc()
            self.error(e.error)

            context = {'error': str(e)}
            self.doRender('error.html', context)
            return
        except DownloadError:
            context = {'error':
                       'iDigi fetch failed - is your gateway turned on?'}
            self.doRender('error.html', context)
            return
        except iDigiError, e:
            context = {'error': str(e)}
            self.doRender('error.html', context)
            return


        # Format variables for template population of gauges

        stup = [(s.channel_name, s) for s in samples]
        samples = dict(stup)

        lth_divs = [GaugeHandler.render_gauge(
                        samples.get('sensor0.light', None)),
                    GaugeHandler.render_gauge(
                        samples.get('sensor0.temperature', None)),
                    ]
        if lth_divs == ['', '']:
            lth_divs = None

        rpm_divs = [GaugeHandler.render_gauge(
                        samples.get('rpm0.current', None)),
                    GaugeHandler.render_gauge(
                        samples.get('rpm0.power', None)),
                    ]
        if rpm_divs == ['', '']:
            rpm_divs = None

        xbr_divs = [GaugeHandler.render_gauge(
                        samples.get('xbr0.light', None)),
                    GaugeHandler.render_gauge(
                        samples.get('xbr0.temperature', None)),
                    ]
        if xbr_divs == ['', '']:
            xbr_divs = None

        ## Miscellaneous state

        power_on = samples.get('rpm0.power_on',
                               Sample(channel_name="dumb", value="Off"))

        power_on = True if power_on.value == 'On' else False

        ## History variables calculations

        # Light history
        light_history = HistoryHandler.render_history('light', user)

        # Temperature history
        temperature_history = HistoryHandler.render_history(
            'temperature', user)

        # Current history
        current_history = HistoryHandler.render_history(
            'current', user)

        ## Render full main page

        context = {'lth_divs': lth_divs,
                   'rpm_divs': rpm_divs,
                   'xbr_divs': xbr_divs,
                   'power_on': power_on,
                   'light_history': light_history,
                   'temperature_history': temperature_history,
                   'current_history': current_history,
                   'user': user
                   }
                    
        self.doRender('index.html', context)
        

# hack to add all "Handler" classes
classes = filter(lambda x: x.find("Handler")>0, locals().keys())
controllers = [ ("/%s"%x[:-7].lower(), locals()[x]) for x in classes ]
def main():
    # use the controllers, plus hardcoded "/" to the login handler
    application = webapp.WSGIApplication([('/',MainHandler)]+controllers , debug=True)
    util.run_wsgi_app(application)


if __name__ == '__main__':
    main()
