############################################################################
#                                                                          #
# Copyright (c)2008, 2010, Digi International (Digi). All Rights Reserved. #
#                                                                          #
# Permission to use, copy, modify, and distribute this software and its    #
# documentation, without fee and without a signed licensing agreement, is  #
# hereby granted, provided that the software is used on Digi products only #
# and that the software contain this copyright notice,  and the following  #
# two paragraphs appear in all copies, modifications, and distributions as #
# well. Contact Product Management, Digi International, Inc., 11001 Bren   #
# Road East, Minnetonka, MN, +1 952-912-3444, for commercial licensing     #
# opportunities for non-Digi products.                                     #
#                                                                          #
# DIGI SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED   #
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          #
# PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, #
# PROVIDED HEREUNDER IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND. #
# DIGI HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,         #
# ENHANCEMENTS, OR MODIFICATIONS.                                          #
#                                                                          #
# IN NO EVENT SHALL DIGI BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,      #
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,   #
# ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF   #
# DIGI HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                #
#                                                                          #
############################################################################


"""\
This module contains the database models using the google appengine ORM
"""

import re
import base64
import logging

from xml.dom import minidom
from google.appengine.api import memcache, urlfetch
from google.appengine.ext import db

from idigi import format_gateway, iDigiRequestError

## Constants

REQUEST_RCI = """<do_command target="idigi_dia">%s</do_command> """

SCI_TMPL = """
<sci_request version="1.0">
  <send_message cache="%s" >
        <targets> <device id="%s"/> </targets>
        <rci_request version="1.1">
 %s
        </rci_request>
  </send_message> 
</sci_request>
"""

class User(db.Model):
    username = db.StringProperty(required=True)
    password = db.StringProperty()
    auth = db.StringProperty() # authentication string for basic encryption.

    gateway = db.StringProperty()
    csrf_id = db.StringProperty()

    logging = db.BooleanProperty(default=False)
    logging_deadline = db.DateTimeProperty()
    
    @staticmethod
    def get_or_create(username, password, gateway):
        user = User.get_by_key_name(username)
        if user is None:
            logging.debug("Creating new user object")
            user = User(key_name=username, username=username)
        logging.debug("User: " + str(user.username))

        if password != user.password or gateway != user.gateway:
            logging.debug("Details mismatch %s:%s" % (password, user.password))
            
            user.password = password
            user.gateway = format_gateway(gateway)
            user.auth = "Basic %s" % base64.encodestring('%s:%s' % (username, 
                                                                password))[:-1]
            result = user.dev_info(device=user.gateway) # Try it, exc. on fail

            if result.status_code != 200:
                raise iDigiRequestError(result.status_code)

            # TODO: Should verify the gateway here before we get much further
            logging.debug("Saving user %s" % user.username)
            user.put()

        return user
    
    def dev_info(self, device="all", cache="true"):
        return self.query("<query_state/>", device_id=device, cache=cache)
    
    def query(self, data, device_id=None, cache='true', path='developer.idigi.com/ws/sci'):
        if device_id is None:
            if not self.gateway:
                raise NoDevice("No Device selected")
            else:
                device_id = self.gateway
        return urlfetch.fetch(url="https://%s" % path,
                              payload=SCI_TMPL % (cache, device_id, data),
                              method=urlfetch.POST, deadline=10,
                              headers={'Authorization': self.auth})


class NoDevice(Exception):
    pass
